
LD  <- function(R,Tau){
##  INPUT variables
####################
##
##  R   : matrix T x (pmax+1)
##      Covariance matrix with R[k,l]=R_k(l-1)
##      Warning: R_k(l) is setting to 0 for l>pk
##  Tau   : matrix T x (pmax+1)
##      Tau[k,l] = 1 if R_k(l-1) is known, 0 if R_k(l-1)
##      corresponds to a gap and 2 if it is such that
##      l > p_k.
##  warning: in this prg the filter are the ones defined in Lambert-Lacroix
##  OUTPUT variables
#####################
##  Structure with fields
##  R   : matrix T x (pmax+1)
##      Covariance matrix with R[k,l]=R_k(l)
##  Beta: matrix T x (pmax+1)
##      Partial correlation matrix with Beta[k,l]=Beta_k(l)
##

#Determine T, pmax, pmin and p the order vector
T <- dim(R)[1]
pmax <- dim(R)[2]-1
p <- rep(pmax,T)-apply((Tau == 2),1,sum)

#Initialize
Beta <- matrix(0,nrow=T,ncol=(pmax+1))
Beta[,1] <-R[,1]
Raux <- NULL
af <- matrix(rep(1,T),nrow=T)
ab <- matrix(rep(1,T),nrow=T)
sigma2f <- R[,1]
sigma2b <- c(sigma2f[T],sigma2f[1:(T-1)])


for (n in .seqinc(1,pmax))  # change by Georgi for this to work when pmax=0.
                            # was: for (n in 1:pmax)
 {#Update Raux (vector of Rk(n))
  Raux <- cbind(R[,(n+1)],Raux)

  #Compute A (vector of Ak(n))
  A <- apply(Raux*af,1,sum)
  if (sum(n>p)>0)
    {R[n>p,n] <- -A[n>p]
     A[n>p] <- 0}

  #Compute partial correlation
  Beta[,(n+1)] <- A/sqrt(sigma2f*sigma2b)

  # #Stop the programme if Beta is outside ]-1,+1[
  # if (max(abs(Beta[,-1]))>=1)
  #  {warning("R is not positive definite")
  #   return(Beta)}

  #Stop the programme if Beta is outside ]-1,+1[
  if (max(abs(Beta[,-1]))>=1){    # change by Georgi
    # print("R is not positive definite")
    # return(Beta)
    return(NA)
  }

  #Update filter
  afn <- -Beta[,(n+1)]*sqrt(sigma2f/sigma2b)
  abn <- -Beta[,(n+1)]*sqrt(sigma2b/sigma2f)
  afaux <- af
  abaux <- ab
  if (n>1)
   {af[,2:n]<-afaux[,2:n]+sweep(abaux[,1:(n-1),drop=FALSE],1,afn,"*")
    ab[,1:(n-1)]<-abaux[,1:(n-1)]+sweep(afaux[,2:n,drop=FALSE],1,abn,"*")}
  af <- cbind(af,afn)
  ab <- cbind(abn,ab)

  #Update variances
  vaux <- (1-Beta[,(n+1)]^2)
  sigma2f <- vaux*sigma2f
  sigma2b <- vaux*sigma2b

  #Build Raux, sigma2b and ab for the next step
  if (n==1)
    Raux <- matrix(c(Raux[T,],Raux[1:(T-1),]),nrow=T)
  if (n>1)
    Raux <- rbind (Raux[T,],Raux[1:(T-1),])
  sigma2b <- c(sigma2b[T],sigma2b[1:(T-1)])
  ab <- rbind (ab[T,],ab[1:(T-1),])

}

return(list(R=R,Beta=Beta,sigma2f=sigma2f))
}
