LDgrad  <- function(R,Tau){
##  INPUT variables
####################
##
##  R   : matrix T x (pmax+1)
##      Covariance matrix with R[k,l]=R_k(l-1)
##      Warning: R_k(l) is setting to 0 for l>pk
##  Tau   : matrix T x (pmax+1)
##      Tau[k,l] = 1 if R_k(l-1) is known, 0 if R_k(l-1)
##      corresponds to a gap and 2 if it is such that
##      l > p_k.
##  warning: in this prg the filter are the ones defined in Lambert-Lacroix
##  OUTPUT variables
#####################
##  Structure with fields
##      gradsigma2fsigma2f
##      sigma2f

#Determine T, pmax, pmin and p the order vector
T <- dim(R)[1]
pmax <- dim(R)[2]-1
p <- rep(pmax,T)-apply((Tau == 2),1,sum)

#Determine the number of gaps
indexgaps <- which(Tau == 0, arr.ind=TRUE)
nbgaps <- dim(indexgaps)[1]
locgaps <- matrix(0,T,(pmax+1))
locgaps[indexgaps]=1:nbgaps
cptgaps <- 0

#Initialize
#for LD
Beta <- matrix(0,nrow=T,ncol=(pmax+1))
Beta[,1] <-R[,1]
Raux <- NULL
af <- matrix(rep(1,T),nrow=T)
ab <- matrix(rep(1,T),nrow=T)
sigma2f <- R[,1]
sigma2b <- c(sigma2f[T],sigma2f[1:(T-1)])
#for gradient
locgapsaux <- NULL
gradsigma2f <- vector("list",nbgaps)
gradsigma2b <- vector("list",nbgaps)
gradaf <- vector("list",nbgaps)
gradab <- vector("list",nbgaps)
for (igaps in 1:nbgaps)
 {gradsigma2f[[igaps]] <- rep(0,T)
  gradsigma2b[[igaps]] <- rep(0,T)
  ncol <- (indexgaps[igaps,2]-2)
  if (ncol==0)
   {gradaf[[igaps]] <- numeric(0)
    gradab[[igaps]] <- numeric(0)}
  if (ncol>0)
   {gradaf[[igaps]] <- matrix(0,nrow=T,ncol=ncol)
    gradab[[igaps]] <- matrix(0,nrow=T,ncol=ncol)}
 }


for (n in 1:pmax)
 {#Update Raux (vector of Rk(n))
  Raux <- cbind(R[,(n+1)],Raux)

  #Compute A (vector of Ak(n))
  A <- apply(Raux*af,1,sum)
  if (sum(n>p)>0)
    {R[n>p,n] <- -A[n>p]
     A[n>p] <- 0}

  #Compute partial correlation
  Beta[,(n+1)] <- A/sqrt(sigma2f*sigma2b)

  #Update filter
  afn <- -Beta[,(n+1)]*sqrt(sigma2f/sigma2b)
  abn <- -Beta[,(n+1)]*sqrt(sigma2b/sigma2f)
  afaux <- af
  abaux <- ab
  if (n>1)
   {af[,2:n]<-afaux[,2:n]+sweep(abaux[,1:(n-1),drop=FALSE],1,afn,"*")
    ab[,1:(n-1)]<-abaux[,1:(n-1)]+sweep(afaux[,2:n,drop=FALSE],1,abn,"*")}
  af <- cbind(af,afn)
  ab <- cbind(abn,ab)


  #Compute if necessary some derivative
  locgapsaux <- cbind(locgaps[,(n+1)],locgapsaux)
  cpt <- max(locgaps[,1:(n+1)])
  if (cpt>0) {
  for (igaps in 1:cpt)
   {#Update the different gradients
    #gradA
    if (n==1)
     gradA <- apply(afaux*((locgapsaux==igaps)+0),1,sum)
    if (n>1)
     gradA <- apply(Raux[,-1]*gradaf[[igaps]],1,sum)+apply(afaux*((locgapsaux==igaps)+0),1,sum)
    #derivative of filters
    gradafn <- -(sigma2b*gradA-A*gradsigma2b[[igaps]])/sigma2b^2
    gradabn <- -(sigma2f*gradA-A*gradsigma2f[[igaps]])/sigma2f^2
    gradafaux <- gradaf[[igaps]]
    gradabaux <- gradab[[igaps]]
    if (n>1)
     {gradaf[[igaps]]<-gradafaux+sweep(as.matrix(abaux[,1:(n-1)]),1,gradafn,"*")+sweep(as.matrix(gradabaux),1,afn,"*")
      gradab[[igaps]]<-gradabaux+sweep(as.matrix(afaux[,2:n]),1,gradabn,"*")+sweep(as.matrix(gradafaux),1,abn,"*")}
    gradaf[[igaps]] <- cbind(gradaf[[igaps]],gradafn)
    gradab[[igaps]] <- cbind(gradabn,gradab[[igaps]])
    #derivative of variances
    gradsigma2f[[igaps]] <- gradsigma2f[[igaps]] -2*A*gradA/sigma2b+A^2*gradsigma2b[[igaps]]/sigma2b^2
    gradsigma2b[[igaps]] <- gradsigma2b[[igaps]] -2*A*gradA/sigma2f+A^2*gradsigma2f[[igaps]]/sigma2f^2
    #Build gradsigma2b[[igaps]] and gradab[[igaps]] for the next step
    gradsigma2b[[igaps]] <- c(gradsigma2b[[igaps]][T],gradsigma2b[[igaps]][1:(T-1)])
    gradab[[igaps]] <- rbind (gradab[[igaps]][T,,drop=FALSE],gradab[[igaps]][1:(T-1),,drop=FALSE])
   }
  }


  #Update variances
  vaux <- (1-Beta[,(n+1)]^2)
  sigma2f <- vaux*sigma2f
  sigma2b <- vaux*sigma2b

  #Build Raux, sigma2b and ab for the next step
  if (n==1)
    {Raux <- matrix(c(Raux[T,],Raux[1:(T-1),]),nrow=T)
     locgapsaux <- matrix(c(locgapsaux[T,],locgapsaux[1:(T-1),]),nrow=T)}
  if (n>1)
    {Raux <- rbind (Raux[T,],Raux[1:(T-1),])
     locgapsaux <- rbind (locgapsaux[T,],locgapsaux[1:(T-1),])}
  sigma2b <- c(sigma2b[T],sigma2b[1:(T-1)])
  ab <- rbind (ab[T,],ab[1:(T-1),])


}

return(list(gradsigma2f=gradsigma2f,sigma2f=sigma2f))
}
