LDhessian  <- function(R,Tau){
##  INPUT variables
####################
##
##  R   : matrix T x (pmax+1)
##      Covariance matrix with R[k,l]=R_k(l-1)
##      Warning: R_k(l) is setting to 0 for l>pk
##  Tau   : matrix T x (pmax+1)
##      Tau[k,l] = 1 if R_k(l-1) is known, 0 if R_k(l-1)
##      corresponds to a gap and 2 if it is such that
##      l > p_k.
##  warning: in this prg the filter are the ones defined in Lambert-Lacroix
##  OUTPUT variables
#####################
##  Structure with fields
##      gradsigma2fsigma2f
##      sigma2f

#Determine T, pmax, pmin and p the order vector
T <- dim(R)[1]
pmax <- dim(R)[2]-1
p <- rep(pmax,T)-apply((Tau == 2),1,sum)

#Determine the number of gaps
indexgaps <- which(Tau == 0, arr.ind=TRUE)
nbgaps <- dim(indexgaps)[1]
locgaps <- matrix(0,T,(pmax+1))
locgaps[indexgaps]=1:nbgaps
cptgaps <- 0

#Initialize
#for LD
Beta <- matrix(0,nrow=T,ncol=(pmax+1))
Beta[,1] <-R[,1]
Raux <- NULL
af <- matrix(rep(1,T),nrow=T)
ab <- matrix(rep(1,T),nrow=T)
sigma2f <- R[,1]
sigma2b <- c(sigma2f[T],sigma2f[1:(T-1)])
#for gradient and hessian
locgapsaux <- NULL
gradsigma2f <- vector("list",nbgaps)
gradsigma2b <- vector("list",nbgaps)
gradaf <- vector("list",nbgaps)
gradab <- vector("list",nbgaps)
gradA <- vector("list",nbgaps)
grad2sigma2f <- vector("list",nbgaps)
grad2sigma2b <- vector("list",nbgaps)
grad2af <- vector("list",nbgaps)
grad2ab <- vector("list",nbgaps)

for (igaps in 1:nbgaps)
 {gradsigma2f[[igaps]] <- rep(0,T)
  gradsigma2b[[igaps]] <- rep(0,T)
  gradA[[igaps]] <- rep(0,T)
  ncol <- (indexgaps[igaps,2]-2)
  if (ncol==0)
   {gradaf[[igaps]] <- numeric(0)
    gradab[[igaps]] <- numeric(0)}
  if (ncol>0)
   {gradaf[[igaps]] <- matrix(0,nrow=T,ncol=ncol)
    gradab[[igaps]] <- matrix(0,nrow=T,ncol=ncol)}

  grad2sigma2f[[igaps]] <- vector("list",igaps)
  grad2sigma2b[[igaps]] <- vector("list",igaps)
  grad2af[[igaps]] <- vector("list",igaps)
  grad2ab[[igaps]] <- vector("list",igaps)

  for (ii in 1:igaps)
    {grad2sigma2f[[igaps]][[ii]] <- rep(0,T)
     grad2sigma2b[[igaps]][[ii]] <- rep(0,T)
     grad2af[[igaps]][[ii]] <- gradaf[[igaps]]
     grad2ab[[igaps]][[ii]] <- gradab[[igaps]]
    }
 }

#Udapte ...old
gradsigma2fold <- gradsigma2f
gradsigma2bold <- gradsigma2b
gradafold <- gradaf
gradabold <- gradab
grad2sigma2fold <- gradsigma2f
grad2sigma2bold <- gradsigma2b
grad2afold <- grad2af
grad2abold <- grad2ab


for (n in 1:pmax)
 {#Update Raux (vector of Rk(n))
  Raux <- cbind(R[,(n+1)],Raux)

  #Compute A (vector of Ak(n))
  A <- apply(Raux*af,1,sum)
  if (sum(n>p)>0)
    {R[n>p,n] <- -A[n>p]
     A[n>p] <- 0}

  #Compute partial correlation
  Beta[,(n+1)] <- A/sqrt(sigma2f*sigma2b)

  #Update filter
  afn <- -Beta[,(n+1)]*sqrt(sigma2f/sigma2b)
  abn <- -Beta[,(n+1)]*sqrt(sigma2b/sigma2f)
  afaux <- af
  abaux <- ab
  if (n>1)
   {af[,2:n]<-afaux[,2:n]+sweep(abaux[,1:(n-1),drop=FALSE],1,afn,"*")
    ab[,1:(n-1)]<-abaux[,1:(n-1)]+sweep(afaux[,2:n,drop=FALSE],1,abn,"*")}
  af <- cbind(af,afn)
  ab <- cbind(abn,ab)


  #Compute if necessary some first order derivative
  locgapsaux <- cbind(locgaps[,(n+1)],locgapsaux)
  cpt <- max(locgaps[,1:(n+1)])
  if (cpt>0) {
  for (igaps in 1:cpt)
   {#Update the different gradients
    #gradA
    if (n==1)
     gradA[[igaps]] <- apply(afaux*((locgapsaux==igaps)+0),1,sum)
    if (n>1)
     gradA[[igaps]] <- apply(Raux[,-1]*gradaf[[igaps]],1,sum)+
                        apply(afaux*((locgapsaux==igaps)+0),1,sum)
    #derivative of filters
    gradafn <- -(sigma2b*gradA[[igaps]]-A*gradsigma2b[[igaps]])/sigma2b^2
    gradabn <- -(sigma2f*gradA[[igaps]]-A*gradsigma2f[[igaps]])/sigma2f^2
    gradafaux <- gradaf[[igaps]] ##remplacer par old
    gradabaux <- gradab[[igaps]]
    if (n>1)
     {gradaf[[igaps]]<-gradafaux+sweep(as.matrix(abaux[,1:(n-1)]),1,gradafn,"*")+
                        sweep(as.matrix(gradabaux),1,afn,"*")
      gradab[[igaps]]<-gradabaux+sweep(as.matrix(afaux[,2:n]),1,gradabn,"*")+
                        sweep(as.matrix(gradafaux),1,abn,"*")}
    gradaf[[igaps]] <- cbind(gradaf[[igaps]],gradafn)
    gradab[[igaps]] <- cbind(gradabn,gradab[[igaps]])
    #derivative of variances
    gradsigma2f[[igaps]] <- gradsigma2f[[igaps]] -2*A*gradA[[igaps]]/sigma2b+
                               A^2*gradsigma2b[[igaps]]/sigma2b^2
    gradsigma2b[[igaps]] <- gradsigma2b[[igaps]] -2*A*gradA[[igaps]]/sigma2f+
                               A^2*gradsigma2f[[igaps]]/sigma2f^2

    #Build gradsigma2b[[igaps]] and gradab[[igaps]] for the next step
    gradsigma2b[[igaps]] <- c(gradsigma2b[[igaps]][T],gradsigma2b[[igaps]][1:(T-1)])
    gradab[[igaps]] <- rbind (gradab[[igaps]][T,,drop=FALSE],gradab[[igaps]][1:(T-1),,drop=FALSE])

    #Update the different second derivatives
    for (ii in 1:igaps)
      {#Update grad2A
       if (n==1)
           grad2A <- rep(0,T)
       if (n>1)
           grad2A <- apply(Raux[,-1]*grad2afold[[igaps]][[ii]],1,sum)+
                      apply((gradafold[[igaps]]*((locgapsaux==ii)+0)[,-1,drop=FALSE]+
                      gradafold[[ii]]*((locgapsaux==igaps)+0)[,-1,drop=FALSE]),1,sum)

       #Update second derivative for filter
       grad2afn <- -2*gradsigma2bold[[igaps]]/sigma2b*gradaf[[ii]][,n]-
                    (gradsigma2bold[[igaps]]*gradA[[ii]]+sigma2b*grad2A-
                     gradA[[igaps]]*gradsigma2bold[[ii]]-
                     A*grad2sigma2bold[[igaps]][[ii]])/sigma2b^2
       grad2abn <- -2*gradsigma2fold[[igaps]]/sigma2f*
                     c(gradab[[ii]][(2:T),1],gradab[[ii]][1,1])-
                     (gradsigma2fold[[igaps]]*gradA[[ii]]+sigma2f*grad2A-
                     gradA[[igaps]]*gradsigma2fold[[ii]]-
                     A*grad2sigma2fold[[igaps]][[ii]])/sigma2f^2
       if (n>1)
        {grad2af[[igaps]][[ii]]<- grad2afold[[igaps]][[ii]]+
                       sweep(as.matrix(abaux[,1:(n-1)]),1,grad2afn,"*")+
                       sweep(as.matrix(gradabold[[igaps]]),1,gradaf[[ii]][,n],"*")+
                       sweep(as.matrix(gradabold[[ii]]),1,gradaf[[igaps]][,n],"*")+
                       sweep(as.matrix(grad2abold[[igaps]][[ii]]),1,afn,"*")
         grad2ab[[igaps]][[ii]]<- grad2abold[[igaps]][[ii]]+
                       sweep(as.matrix(afaux[,1:(n-1)]),1,grad2abn,"*")+
                       sweep(as.matrix(gradafold[[igaps]]),1,c(gradab[[ii]][(2:T),1],
                          gradab[[ii]][1,1]),"*")+
                       sweep(as.matrix(gradafold[[ii]]),1,c(gradab[[igaps]][(2:T),1],
                          gradab[[igaps]][1,1]),"*")+
                       sweep(as.matrix(grad2afold[[igaps]][[ii]]),1,abn,"*")
        }
       grad2af[[igaps]][[ii]] <- cbind(grad2af[[igaps]][[ii]],grad2afn)
       grad2ab[[igaps]][[ii]] <- cbind(grad2abn,grad2ab[[igaps]][[ii]])

       #Update second derivative for variances
       grad2sigma2f[[igaps]][[ii]] <- grad2sigma2fold[[igaps]][[ii]] -
                          2*(gradA[[igaps]]*gradA[[ii]]/sigma2b +
                          A*grad2A/sigma2b-A*gradA[[ii]]*gradsigma2bold[[igaps]]/sigma2b^2)+
                          2*A*gradA[[igaps]]*gradsigma2bold[[ii]]/sigma2b^2+
                          A^2*grad2sigma2bold[[igaps]][[ii]]/sigma2b^2-
                          2*A^2*gradsigma2bold[[ii]]*gradsigma2bold[[igaps]]/sigma2b^3
       grad2sigma2b[[igaps]][[ii]] <- grad2sigma2bold[[igaps]][[ii]] -
                          2*(gradA[[igaps]]*gradA[[ii]]/sigma2f +
                          A*grad2A/sigma2f-A*gradA[[ii]]*gradsigma2fold[[igaps]]/sigma2f^2)+
                          2*A*gradA[[igaps]]*gradsigma2fold[[ii]]/sigma2f^2+
                          A^2*grad2sigma2fold[[igaps]][[ii]]/sigma2f^2-
                          2*A^2*gradsigma2fold[[ii]]*gradsigma2fold[[igaps]]/sigma2f^3

      #Build gradsigma2b[[igaps]] and gradab[[igaps]] for the next step
      grad2sigma2b[[igaps]][[ii]] <- c(grad2sigma2b[[igaps]][[ii]][T],grad2sigma2b[[igaps]][[ii]][1:(T-1)])
      grad2ab[[igaps]][[ii]] <- rbind (grad2ab[[igaps]][[ii]][T,,drop=FALSE],grad2ab[[igaps]][[ii]][1:(T-1),,drop=FALSE])
      }

   }
  }


  #Update variances
  vaux <- (1-Beta[,(n+1)]^2)
  sigma2f <- vaux*sigma2f
  sigma2b <- vaux*sigma2b

  #Build Raux, sigma2b and ab for the next step
  if (n==1)
    {Raux <- matrix(c(Raux[T,],Raux[1:(T-1),]),nrow=T)
     locgapsaux <- matrix(c(locgapsaux[T,],locgapsaux[1:(T-1),]),nrow=T)}
  if (n>1)
    {Raux <- rbind (Raux[T,],Raux[1:(T-1),])
     locgapsaux <- rbind (locgapsaux[T,],locgapsaux[1:(T-1),])}
  sigma2b <- c(sigma2b[T],sigma2b[1:(T-1)])
  ab <- rbind (ab[T,],ab[1:(T-1),])

  #Update  ..old
  gradsigma2fold <- gradsigma2f
  gradsigma2bold <- gradsigma2b
  gradafold <- gradaf
  gradabold <- gradab
  grad2sigma2fold <- grad2sigma2f
  grad2sigma2bold <- grad2sigma2b
  grad2afold <- grad2af
  grad2abold <- grad2ab


}

return(list(sigma2f=sigma2f,gradsigma2f=gradsigma2f,grad2sigma2f=grad2sigma2f))
}
