# formelrly parameter() in parameter.R and paperex.R
pldinverse <- function(beta){
##  INPUT variables
####################
##
##  beta   : matrix T x (pmax+1)
##      pacf matrix with beta[k,l]=beta_k(l-1)
##      beta_k(l) is 0 for l>pk
##  warning: in this prg the filter are the ones defined in Lambert-Lacroix
##  OUTPUT variables
#####################
##  Structure with fields
##  sigma   : vector of length T
##      variances
##  Af: matrix T x (pmax+1)
##      filter
##

#Determine T, pmax, pmin and p the order vector
T <- dim(beta)[1]
pmax <- dim(beta)[2]-1

#Initialize
R <- matrix(0,nrow=T,ncol=(pmax+1))
R[,1] <-beta[,1]
Raux <- matrix(0,nrow=T)
af <- matrix(rep(1,T),nrow=T)
ab <- matrix(rep(1,T),nrow=T)
sigma2f <- R[,1]
sigma2b <- c(sigma2f[T],sigma2f[1:(T-1)])

for (n in .seqinc(1,pmax))  # change by Georgi for this to work when pmax=0.
                            # was: for (n in 1:pmax)
 {

  #Compute A (vector of Ak(n))
  A <- apply(Raux*af,1,sum)

  #Compute R
  R[,(n+1)] <- beta[,(n+1)]*sqrt(sigma2f*sigma2b)-A

  #Update filter
  afn <- -beta[,(n+1)]*sqrt(sigma2f/sigma2b)
  abn <- -beta[,(n+1)]*sqrt(sigma2b/sigma2f)
  afaux <- af
  abaux <- ab
  if (n>1)
   {af[,2:n]<-afaux[,2:n]+sweep(abaux[,1:(n-1),drop=FALSE],1,afn,"*")
    ab[,1:(n-1)]<-abaux[,1:(n-1)]+sweep(afaux[,2:n,drop=FALSE],1,abn,"*")}
  af <- cbind(af,afn)
  ab <- cbind(abn,ab)

  #Update variances
  vaux <- (1-beta[,(n+1)]^2)
  sigma2f <- vaux*sigma2f
  sigma2b <- vaux*sigma2b

  #Build Raux, sigma2b and ab for the next step
  Raux[,1] <- R[,(n+1)]
  Raux <- cbind(rep(0,T),Raux)
  Raux <- rbind (Raux[T,],Raux[1:(T-1),])
  sigma2b <- c(sigma2b[T],sigma2b[1:(T-1)])
  ab <- rbind (ab[T,],ab[1:(T-1),])

}

return(list(R=R,Af=af,sigma2f=sigma2f))
}
