# source("LD.r")
# source("LDgrad.r")
# source("LDhessian.r")
# source("myhessian.r")

sigma2.to.enthess <- function(sigma2,gradsigma2,grad2sigma2){
  pcT <- length(sigma2)
  nbgaps <- length(gradsigma2)
  if(nbgaps==1){
    enthessian <- sum(grad2sigma2[[1]][[1]]/sigma2-(gradsigma2[[1]]/sigma2)^2) / pcT
  }else{
    enthessian <- matrix(0,ncol=nbgaps,nrow=nbgaps)
    gradsigma2norm <- sapply(gradsigma2, function(x) x/sigma2)
    for (igaps in (1:nbgaps))
       {enthessian[igaps,1:igaps] <- sapply(grad2sigma2[[igaps]], function(x) sum(x/sigma2))-
        apply(sweep(gradsigma2norm[,1:igaps,drop=FALSE],1,gradsigma2norm[,igaps],"*"),2,sum)
       }
    enthessian <- enthessian+t(enthessian)
    diag(enthessian) <- diag(enthessian)/2
    enthessian <- enthessian / pcT
  }
return(enthessian)
}



.armijo <- function(f,x,h,alpha,beta,grad,k0=0,kmax=100   # alpha in (0,1/2), beta in (0,1)
                   ){

  fx <- f(x)
  wrk <- alpha * (h %*% grad)

  # if(wrk>=0) changed on 9/1/2009 to wrk>0,
  # sometimes grad is the 0 vector, how does this happen numerically?
  # (it seems that this can happen in the p.d. case!)
  if(wrk>0){
    print("direction of increase in .armijo!")
    browser()
  }

  if(k0==0  &&  f(x + h ) - fx <= wrk )
    return(k0)

  # if(k0>100) k0 <- 0

  k <- k0
  bk <- beta^k
  while(k>0  &&  f(x + bk * h ) - fx <= bk * wrk  ){
    k <- k-1
    bk <- bk/beta;
  }
  k <- k + 1  # note: after this command bk = beta^(k-1)

  if(k>k0){
    # wrksmallflag <- abs(wrk) < 10^(-15)  # arbitrary choice here, see below too.
    # if(wrksmallflag){
    #   k <- 40
    #   bk <- beta^39
    # }
    while( { bk <- bk*beta;
             fxbh <- f(x + bk*h);
             is.na(fxbh)  ||
             fxbh - fx > bk * wrk + 10^(-16) #|| #adding a small  value to avoid infinite loop
              # (wrksmallflag  &&  fxbh - fx >= 0) # my correction
           }){
      k <- k+1
    }
  }

  k
}

mynorm <- function(x){
  sqrt(sum(x^2))
}

.update.optimhist <- function(x,fx,gradfx,hessfx,cur){
    n <- length(x)
    if(missing(cur)){                                                # start a new history log
        curnames <- c(paste("x",1:n,sep=""),                 # x
                      "fx",                                  # f(x)
                      paste("gradfx",1:n,sep=""),            # gradientf(x)
                      if(n==1)                               # Hessianf(x)
                      "H1.1"
                      else
                      paste("H",
                            row(hessfx)[row(hessfx)>=col(hessfx)],
                            ".",
                            col(hessfx)[row(hessfx)>=col(hessfx)],
                            sep="")
                      )
        cur <- matrix(NA,nrow=0,ncol=length(curnames) )   # n+1+n+n*(n+1)/2 columns
        colnames(cur) <- curnames
    }
    rbind(cur, c(x,fx,gradfx,
                 if(n==1) hessfx else hessfx[lower.tri(hessfx,diag=TRUE)]
                 ))
}


.stabilNewton.make.value <-
    function(minimum, estimate = numeric(0),
                      gradient = as.numeric(rep(NA,length(estimate))),
                      hessian = matrix(NA,nrow=length(estimate), ncol=length(estimate)),
                      iterations = if(length(estimate)==0) 0 else NA,
                      cntgrad    = if(length(estimate)==0) 0 else NA,
                      cnthes     = if(length(estimate)==0) 0 else NA,
                      karmijo = NA,
                      optimhistory = list()
             ){

        list(  minimum      = minimum
             , estimate     = estimate
             , gradient     = gradient
             , hessian      = hessian
             , iterations   = iterations
             , cntgrad      = cntgrad
             , cnthes       = cnthes
             , karmijo      = karmijo
             , optimhistory = optimhistory
             )
}

stabilNewton <- function(f, x
                         , alpha = 10^(-4)
                         , beta=0.9         # may be smaller with good initial values
                         , xvaltol=10^(-6)        #10^(-6) = 10^2 sqrt(eps)
                         , gradtol=10^{-12}       #10^4*eps
                         , gradnormtol=10^{-8}
                         , itermax=100
                         , fonly
                         , choosedirtol = 10^(-12)
                         , betaflag = TRUE  # allow decrease of beta during minimisation.
                         ){
  # print("!!! ???")
  if(missing(fonly))
    fonly <- f

  # browser()

  cntgrad <- 0
  cnthes <- 0

  requiregrad <- FALSE

  flag.optimhistory <- TRUE  # add an argument fo the function to control this! ???

  res <- list()
  karmijo <- matrix(0,nrow=0,ncol=2)
  k <- 0
  # print("!!! ???")

  iter = 0
  # print("!!! ???")
  fx   <- f(x)   # as for nlm(), for now
  # print("!!! ???")
  grad <- attr(fx,"gradient")
  # print("!!! ???")
  h    <- attr(fx,"hessian")
  # curhistory.names <- c("f","gradient","hessian", paste("Rg",1:length(x),sep=""))
  # curhistory <- matrix(c(fx,grad,h,x),nrow=1,dimnames=list(NULL,curhistory.names))
  # print("6!!! ???")
  if(flag.optimhistory)
      curhistory <- .update.optimhist(x,fx,grad,h)                     # start the history log

  # print("!!! ???")
  while(iter <= itermax){
    iter <- iter + 1

    newtondir <- solve(h,-grad)
    slope <- as.numeric(newtondir %*% grad)    # negative if h is positive definite

    flagA <- slope > choosedirtol *  mynorm(newtondir)*mynorm(grad)

    eig <- eigen(h, symmetric = TRUE, only.values = TRUE)   # hcond <- kappa(h)

    # print("")


    flagB <- eig$values[length(eig$values)] < 10^(-8) ||
             eig$values[1]/eig$values[length(eig$values)] > 10^7

    # print("$$$")

    if(!identical(flagA,flagB)){
      cat(!identical(flagA,flagB),": ", "flagA is ", flagA, ", while flagB is ", flagB, "\n")
    }

    flag <- flagA

    v <- if(flag  || requiregrad){
           cntgrad <- cntgrad+1
           -grad
         }else{
           cnthes <- cnthes+1
           newtondir
         }

    k <- .armijo(fonly,x,v,alpha,beta,grad,k)

    if(k==0 && betaflag)     # reduce beta if successful Newton step, my heuristic
      beta <- max(0.1,beta/2)



    karmijo <- rbind(karmijo, c(flag+requiregrad,k))

    xold <- x
    fxold <- fx
    gradold <- grad

    x <- x + beta^k * v
    fx <- f(x)
    grad <- attr(fx,"gradient")
    h    <- attr(fx,"hessian")

    # curhistory <- rbind(curhistory,c(fx,grad,h,x))
    if(flag.optimhistory)
        curhistory <- .update.optimhist(x,fx,grad,h,curhistory)


    # cat("numerical gradient: ", try((fx-fxold)/(x-xold)) , "\n")
    # cat("analytical gradient: ", grad , "\n\n")

    dxcrit <- max(  abs(x-xold) / pmax(abs(xold),10^(-12))  )  # 10^(-12) = eps^(3/4)
    gradcrit <- max( abs(grad * x) / max(abs(fx),10^(-12)) )

    # if( dxcrit < xvaltol || gradcrit < gradtol )
    #   browser()  # for testing

    # print("%%%%")

    if( dxcrit < xvaltol || gradcrit < gradtol ){
      # if(cntgrad==0){    # checked if forcing at least one gradient step inproves--> not.
      #   requiregrad <- TRUE
      # }else
        break
    }
  }
  if(iter>itermax)
    print("max number of iterations reached, convergence may not have been achieved.")

  fx <- f(x)
  .stabilNewton.make.value(  estimate     = x
                           , karmijo      = karmijo
                           , iterations   = iter
                           , minimum      = fx
                           , gradient     = attr(fx,"gradient")
                           , hessian      = attr(fx,"hessian")
                           , cntgrad      = cntgrad
                           , cnthes       = cnthes
                           , optimhistory = curhistory
                           )
}

sigma2.to.ent <- function(sigma2){
  # print("sigma2 is: ");   print(sigma2)
  # print(sigma2)
  if(any(is.nan(sigma2)) || any(is.na(sigma2)) || any(sigma2<0))
    browser()
  sum(log(sigma2))/length(sigma2)     # well, up to a constant
}

sigma2.to.entgrad <- function(sigma2,gradsigma2){
  # print("gradsigma2 is: ");   print(gradsigma2)
  # print("sigma2 is: ");   print(sigma2)
  # wrk <- sapply(gradsigma2, function(x) x) # each element becomes a column.
  pcT <- length(sigma2)
  if(length(gradsigma2)==1){
    wrk <- sum(gradsigma2[[1]]/sigma2)/pcT
  }else{
    wrk <- sapply(gradsigma2, function(x) sum(x/sigma2)/pcT )
  }
  # print("wrk is: "); print(wrk);
  wrk
}


.seqinc <- function(from,to){                   # returns a length(0) numeric vector if from>to
                                        #   useful for "for" loops
    seq(from,length.out=max(0,to-from+1))    # do we need "by" or other arguments?
  }


.shiftleft <- function(x,k=1){                           # works only for 0 <= k <= length(x) !
   ind <- .seqinc(1,k)   # basically ind <- 1:k, but empty vector if k==0 to work for k=0.
   c(x[-ind],x[ind])
}

.adjust.p.orders <- function(p) {  # Sophie's condition
    kmax <- which.max(p);
    ptilde <- numeric(length(p))

    pold <- ptilde[kmax] <- p[kmax]
    for(k in rev( .shiftleft(1:length(p),kmax) )[-1]){
        pold <- ptilde[k] <- max(p[k],pold-1)
    }
    ptilde
}


f.pcentropy <- function(r,tau,method, autoshrink=TRUE){ # 15/03/2009 autoshrink(new)
  ncklimit <- 100   # limit on ck iterations
  ckvec <- numeric(0)
  meth <- method
  entropy <- numeric(0)
  optimhistory <- list()
  nf <- 0

  tauloctmp <- if(!is.list(tau))      # actually, needs to check that it is =="matrix"
                   tau
               else
                   matrix(1,nrow=nrow(r),ncol=ncol(r))

  if(is.list(tau)){                   # (season,lag) pairs here for the gaps
      for(indx in tau){
          tauloctmp[indx[1],indx[2]+1 ] <- 0
      }
  }

                                  # assummng dimensions  of tauloctmp do not exceed those of r
  tauloctmp[  is.na(r[1:nrow(tauloctmp),1:ncol(tauloctmp)])
            & tauloctmp != 2
            ] <- 0

  print("tauloctmp is: ")
  print(tauloctmp)

  if(autoshrink){
                          # for each row of tauloctmp, after a 2, all elements are equal to 2.
                          # (this is what LD and related f. expect)
      p <- max.col(tauloctmp==1,"last") - 1  # ponezhe 1st col e za lag 0 !!!

      tauloctmp[col(tauloctmp) > p + 1  ] <- 2         # replace with 2's

      print("tauloctmp is: "); print(tauloctmp)

      p <- .adjust.p.orders(p)                    # now modify p to satisfy Sophie's condition.

      print("p is "); print(p)

      tauloctmp[col(tauloctmp) > p+1] <- 2                  # this is for LD, LDgrad, etc.
      tauloctmp[col(tauloctmp) <= p+1 & tauloctmp==2] <- 0  # some 2's may have become 0's

      print("tauloctmp is: "); print(tauloctmp)
  }else{
      p <- rep(ncol(tauloctmp)-1, nrow(tauloctmp) )
  }

  rloc <- r
  tauloc <- tauloctmp
  if(autoshrink  &&  1+max(p) < ncol(r) ){
      rloc   <-   rloc[,1:(1+max(p)),drop=FALSE]
      tauloc <- tauloc[,1:(1+max(p)),drop=FALSE]
  }

  gappos <- tauloc==0            # Note: trailing missing values are not gaps!
                               # ??? sinchronisiray s podredbata na Sophie!

  x0 <- rloc[gappos]  # ??? dali da ne slozhi prosto nuli?   b) ??? replace NA with 0's?
  rloc0 <- rloc[,1]


  # print("kiki")

  f0 <- function() x0
  f <- function(x){
    nf <<- nf+1
    rloc[gappos] <- x
    # cat("x is: " ) ; print(x)
    u <- LD(rloc,tauloc)

    if(class(u)=="list"){
      entro   <- sigma2.to.ent(u$sigma2f)
    }else{
      entro <- -100
    }

    res <- entro
    - res
  }
  fg <- function(x){
    nf <<- nf+1
    rloc[gappos] <- x
    # cat("x is: " ) ; print(x)
    # cat("rloc is: " ) ; print(rloc)
    # cat("tauloc is: " ) ; print(tauloc)

    u <- LDgrad(rloc,tauloc)
    entro   <- sigma2.to.ent(u$sigma2f)
    gradient <- sigma2.to.entgrad(u$sigma2f,u$gradsigma2f)

    # print("grad is: "); print(gradient)

    res <- - entro
    attr(res, "gradient") <- - gradient

    res
  }
  fgh <- function(x){
    # print("fgh !!! ???")

    nf <<- nf+1
    rloc[gappos] <- x
    # cat("x is: " ) ; print(x)
    # cat("rloc is: " ) ; print(rloc)
    # cat("tauloc is: " ) ; print(tauloc)

    # print("fgh !!! ???")

    # for testing: slagam v try() ili nesto takova. ???
    u <- LDhessian(rloc,tauloc)
    # print("fgh !!! ???")
    entro   <- sigma2.to.ent(u$sigma2f)
    gradient <- sigma2.to.entgrad(u$sigma2f,u$gradsigma2f)
    hessian <- sigma2.to.enthess(u$sigma2f,u$gradsigma2f,u$grad2sigma2f)

    # cat("eigenvalues of hessian: ", eigen(hessian)$values,"\n")
    # cat("condition number of hessian: ", kappa(hessian), "\n")
    # print(gradient)

    # browser()

    res <- - entro
    attr(res, "gradient") <- - gradient
    attr(res, "hessian") <- - hessian

    entropy <<- c(entropy,entro)

    res
  }
  pdq <- function(x){
    nf <<- nf+1
    rloc[gappos] <- x
    u <- LD(rloc,tauloc)
    class(u)=="list"  && max(abs(u$Beta[,-1]))<1
  }

  makepd <- function(x,c0){
    if(missing(c0))
      c0 <- max(rloc[,1])  # arbitrary choice here
    cnew <- 0
    while( !pdq(x) ){
      cnew <- cnew+c0
      rloc[,1] <<- rloc0 + cnew
    }
    ckvec <<- c(cnew,ckvec)
  }

  mesolve <- function(f,xinit,...){

      if(missing(f))         # added on 19/03/2009, not tested.
          f <- fgh


      print("sum(gappos) is: ");  print(sum(gappos))
      print("gappos is: ");  print(gappos)

      if(sum(gappos)==0){          # obrabotka na sluchaya s nula gaps.
          print("no gaps to fill!")
          u <- LD(rloc,tauloc)
          print("after LD")

          if(class(u)=="list"){                        # ??? pomisli za nesto po-dobro!
              entropy   <<- sigma2.to.ent(u$sigma2f)
          }else{
              entropy <<- NaN
          }
                                                             # imitate output of stabilNewton.
          wrk <- .stabilNewton.make.value(  minimum = -entropy
                                          # ??? pd, nnd atrributes?
                                          )
          return(wrk)
      }
      # else do optimisation


    if(is.numeric(xinit) && length(xinit)==1)   # init with constant, usually 0.
      x0[] <<- xinit
    else if(is.numeric(xinit)){
        if(length(x0) == length(xinit))
            x0 <<- xinit
        else{
            warning("xinit is of wrong length.")
            if(length(x0) < length(xinit))
                x0[] <- xinit[1:length(x0)]
            else
                warning("ignoring xinit.")
        }
    }

    if(!all(complete.cases(x0)))               # replace NA's with 0's.
      x0[is.na(x0) || is.nan(x0)] <- 0

    makepd(x0)
    x <- x0

    cat("starting with ck=",ckvec[1],"\n")
    if(meth=="nlm")
      wrk <- nlm(f,x,...)
    else{
        # print("uf")
      wrk <- stabilNewton(f,x,...)
      optimhistory <<- c(list(wrk$optimhistory),optimhistory)
    }
    nck <- 1
    ckdamp <- 0.95
    ck <- 0
    cat("starting while loop...\n")
    while(rloc[1,1]!=rloc0[1]  &&  nck <= ncklimit  ){
      ckold <- ck
      nck <- nck+1
      # print(nck)
      if(nck>10) # arbitrary constants here, data is probably very close to not p.d.
        ckdamp <- 0.1
      x <- wrk$estimate
      # print("ha ha")
      rloc[gappos] <<- x

      tmp <- rloc[1,1] - rloc0[1]
      # print("risuva")

      # if(!pdq(x+tmp))
      #   print("kiki")
      # tmp2 <- x+tmp
      # if(!pdq(tmp2))
      #   browser()

      # cat("tmp is: ",tmp,"\n")

      rloc[,1] <<- rloc0
      ii <- 0
      ck <- 0
      goon <- TRUE
      while(!pdq(x)){
        ii <- ii+1
        ck <- tmp*(1-ckdamp^ii)
        rloc[,1] <<- rloc0 + ck
        # cat("ii: ", ii, ", ck: ", ck, "\n")
      }
      ckvec <<- c(ck,ckvec)
      # print(x)
      # print(f)
      ###cat("trying ck=",ckvec[1],",\tdecreased by ",ckvec[2]-ckvec[1], " from last ck.","\n")
      if(meth=="nlm")
        wrk <- nlm(f,x,...)
      else{
        wrk <- stabilNewton(f,x,...)
        optimhistory <<- c(list(wrk$optimhistory),optimhistory)
      }
      # print("Optim history is:")
      # print(optimhistory)

      if( ck-ckold == 0){
        goon <- FALSE
        break
      }
    }
    if(nck>ncklimit){
      attr(wrk,"warn:") <- "ncklimit reached"
      attr(wrk,"is.pd") <- FALSE
      attr(wrk,"is.nnd") <- if( ck < 10^(-10) )
                              TRUE
                            else
                              FALSE
    }else{ # ck converged
      if( ck == 0 ){
        attr(wrk,"is.pd") <- TRUE
        attr(wrk,"is.nnd") <- FALSE
      }else if( ck >= 10^(-10) ){   # converged to a positive value
        attr(wrk,"is.pd") <- FALSE
        attr(wrk,"is.nnd") <- FALSE
      }else if( ck < 10^(-10) ){  # seems to converge to zero
       attr(wrk,"is.pd") <- FALSE
       attr(wrk,"is.nnd") <- TRUE
      }else{
        stop("this is impossible!")
      }
    }
    wrk
  }

  ftau <- function() tauloc                 # may be shorter than tauloctmp if autoshrink=TRUE
  ftaufull <- function() tauloctmp
  fent <- function() entropy
  fnf <- function() nf
  fckvec <- function()       rev(ckvec)                    # rev() to obtain natural ordering.
  totalhistory <- function() rev(optimhistory)
  fmeorder <- function() p             # 15/03/2009
  findexgaps <- function() .index.acfgaps(tauloc) #needs to be the same tau given to LDhessian
  ffreelags <- function() .index.acfunavail(tauloc) #has to be the same tau given to LDhessian

  list(mesolve=mesolve
       , f=f, fg=fg, fgh=fgh, x0=f0
       , pdq=pdq, makepd=makepd
       , ftau=ftau, ftaufull = ftaufull
       , findexgaps = findexgaps, ffreelags = ffreelags
       , fent=fent
       , p=fmeorder
       , fnf=fnf
       , fckvec=fckvec
       , totalhistory=totalhistory
       )
}

