pcme <- function(r,tau,x0=NULL,method="stabilNewton",gradient=TRUE,hessian=FALSE,
                 autoshrink=TRUE,...){
    flist <- f.pcentropy(r,tau,method=method,autoshrink=autoshrink)
    if(method=="nlm"){
        f <- if(!gradient)
                 flist$f
             else if(!hessian)
                 flist$fg
             else
                 flist$fgh

        res <- flist$mesolve(f,x0,...)
    }else{
        f <- flist$fgh
        fonly <- flist$f
        res <- try(flist$mesolve(f,x0,fonly=fonly,...),TRUE)      # try() for testing only ???
    }

    sol_tryerror <-
        if(inherits(res, "try-error"))
            res
        else
            NA

    if(!is.list(res)){   # presumably because of error
        res_entropy <- NA                  # ??? put something more appropriate here?
        res_Rg      <- NULL
        res <- list()                  # ??? dali e podchodyasto?
    }else{
        res_entropy  <- -res$minimum      # since we  minimised -entropy
        res_Rg       <- res$estimate

        res$minimum  <- NULL                     # these will only cause confusion if present.
        res$estimate <- NULL
    }

    res_tau <- flist$ftaufull()                         # dali e pravilno saotvetstvieto s Rg?

    res_gaps <- res_indexgaps <- flist$findexgaps() # res$indexgaps
    if(length(res_Rg)>0){
        res_gaps[,2]       <- res_gaps[,2] - 1 #the 2nd column contains lags, counting from 0.
        colnames(res_gaps) <- c("season","lag")
    }

    res_acf <- res_acforig <- r
    res_acforig[ res_tau != 1 ] <- NA  # missings (gaps or not)

    if(!is.null(res_Rg)){                           # lazy, needs better indicator for failure
        res_acf[res_indexgaps] <- res_Rg  # fill the gaps
                                              # but leave trailing holes (i.e. tau==2), if any
    }

    pmax <- max(flist$p())

    tmpacf <- if(pmax + 1 < ncol(res_acf))
                  res_acf[,1:(pmax+1),drop=FALSE]
              else
                  res_acf

    tmpLD <- LD(tmpacf,res_tau)

    if(length(tmpLD) > 1  || (length(tmpLD) == 1  &&  !is.na(tmpLD))){
        tmpLD <- pldinverse(tmpLD$Beta)
        res_coef   <- tmpLD$Af
        res_sigma2 <- tmpLD$sigma2f
        res_acf    <- tmpLD$R
    }else{                                              # result is NA (check more carefully?)
        res_coef   <- NA          # think of better values here. ???
        res_sigma2 <- NA
        # res_acf - leave it as is in this case.
    }

    rownames(res_acf) <- rownames(res_acforig)
    colnames(res_acf) <- colnames(res_acforig)[1:ncol(res_acf)]
    names(dimnames(res_acf)) <- names(dimnames(res_acforig))

    wrk <- list(  p      = flist$p()
                , sigma2 = res_sigma2
                , coef   = res_coef
                , R      = res_acf   # popalnenata acf
                                     # (samo gaps, NA za ostanalite missings, ako ima takiva).

                , tau    = res_tau                                   # tova e flist$ftaufull()

                , entropy   = res_entropy

                , Rg        = res_Rg
                , gaps      = res_gaps

                , freelags  = flist$ffreelags() # rename to freeindexlags?
                , indexgaps = flist$findexgaps()

                , Rorig     = res_acforig

                , tryerror = sol_tryerror

                , enthist = flist$fent()
                , nf      = flist$fnf()
                , ck      = flist$fckvec()

                , totaloptimhistory = flist$totalhistory()
                , x0 = flist$x0()
                )


    res <- c(res,wrk)  # assumes  res is list, which should be so here.
    class(res) <- "pcmesol"
    res
}

print.pcmesol <- function(x, digits = 4, quote = TRUE, prefix = "", ...)
{
    cat("Maximum entropy completion of the following acf", "\n")
    print(x$Rorig)
    cat("\n")
    cat("The maximum entropy model is PAR(", x$p, ")","\n")
    cat("The ME completed R is", "\n")
    print(x$R)
    cat("\n")
    cat("Optimal values of the acf.", "\n")
    if(length(x$gaps>0)){
        cat("(a) for gaps:", "\n")
        mevalues <- cbind(x$gaps,x$Rg)
        colnames(mevalues) <- c("season", "lag", "Rsl")
        print(cbind(x$gaps,x$Rg))
    }else
        cat("\t(There were no gaps.)\n")

    cat("(b) for other missings (non-gaps):", "\n")
    print("??? to be done...")

    cat("\n")

    # cat("\n")
    # cat(strwrap(x$method, prefix = "\t"), sep="\n")
    # cat("\n")
    # cat("data: ", x$data.name, "\n")
    # out <- character()
    # if(!is.null(x$statistic))
    #     out <- c(out, paste(names(x$statistic), "=",
    #     		    format(round(x$statistic, 4))))
    # if(!is.null(x$parameter))
    #     out <- c(out, paste(names(x$parameter), "=",
    #     		    format(round(x$parameter, 3))))
    # if(!is.null(x$p.value)) {
    #     fp <- format.pval(x$p.value, digits = digits)
    #     out <- c(out, paste("p-value",
    #     		    if(substr(fp,1,1) == "<") fp else paste("=",fp)))
    # }
    # cat(strwrap(paste(out, collapse = ", ")), sep="\n")
    # if(!is.null(x$alternative)) {
    #     cat("alternative hypothesis: ")
    #     if(!is.null(x$null.value)) {
    #         if(length(x$null.value) == 1) {
    #     	alt.char <-
    #     	    switch(x$alternative,
    #     		   two.sided = "not equal to",
    #     		   less = "less than",
    #     		   greater = "greater than")
    #     	cat("true", names(x$null.value), "is", alt.char,
    #     	    x$null.value, "\n")
    #         }
    #         else {
    #     	cat(x$alternative, "\nnull values:\n")
    #     	print(x$null.value, ...)
    #         }
    #     }
    #     else cat(x$alternative, "\n")
    # }
    # if(!is.null(x$conf.int)) {
    #     cat(format(100 * attr(x$conf.int, "conf.level")),
    #         "percent confidence interval:\n",
    #         format(c(x$conf.int[1], x$conf.int[2])), "\n")
    # }
    # if(!is.null(x$estimate)) {
    #     cat("sample estimates:\n")
    #     print(x$estimate, ...)
    # }
    cat("\n")
    invisible(x)
}
