\name{pcme.tests}
\alias{pcme.tests}
\alias{pcme.test1}
\alias{pcme.testcombn}
\title{ Solve maximum entropy completion problems on subsets }
\description{
  Given a periodic autocovariance function, declare subsets of
  coefficients to be missing and maximise the entropy. Mainly for testing and examples.
}
\usage{
pcme.test1(r, ...)
pcme.testcombn(r, w = 1, ...)
}
\arguments{
  \item{r}{ the periodic autocovariance function, a numeric matrix.}
  \item{w}{ size of gaps subsets, a numeric vector, see Details. }
  \item{\dots}{ parameters to be passed to the entropy maximisation function.}
}
\details{
  \code{pcme.test1} solves a sequence of maximum entropy (ME) problems with
  one gap. The gap is set to  each  of the non-zero lags, in  turn.

  \code{pcme.testcombn} solves a sequence of maximum entropy problems with
  number of gaps given by \code{w}. All subsets of \code{w} elements
  are considered. If \code{w} is a vector, then this is  done for each
  element of \code{w}. Subsets of all lengths may be requested by
  setting \code{w="all"}.
}
\value{
  For \code{pcme.test1}, a matrix whose \eqn{(s,l)}th element is the
  entropy of the ME solution when the gap is set to \eqn{(s,l)}.

  For \code{pcme.testcombn}, a list of the entropies. Each entropy has
  attibutes \code{gaps} and \code{gapslist} giving the gaps as a matrix
  and list, respectively.
}
\references{
  Boshnakov, Georgi  and Lambert-Lacroix, Sophie (2009?)
  Maximum entropy for periodically correlated processes from
  nonconsecutive autocovariance coefficients.
  \emph{J. Time Series Anal.} (to appear)
}
\author{ Georgi Boshnakov }
\note{
  We used these functions as a basis for bulk testing.

  Setting \code{w="all"} is  convenient but there may be many, many
  subsets to go through...
}
\seealso{ \code{\link{pcme}}, \code{\link{pcme-package}} }
\examples{
data(pcme.paperex)
resex1   <- pcme.test1(pcme.paperex$rex1)
resexAll <- pcme.testcombn(pcme.paperex$rex1,"all")

data(pcme.someex)
pcme.testcombn(pcme.someex$r1,1)
pcme.testcombn(pcme.someex$r1,1:2)
resr1 <- pcme.testcombn(pcme.someex$r1,"all")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ misc }
