\name{pld}
\alias{LD}
\alias{LDgrad}
\alias{LDhessian}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Compute partial autocorrelations, forward variances and their derivatives. }
\description{ Compute partial autocorrelations, forward variances, and
  their first and second  derivatives using the periodic Levinson Durbin algorithm.
}
\usage{
LD(R, Tau)
LDgrad(R, Tau)
LDhessian(R, Tau)
}
\arguments{
  \item{R}{ autocovariances, see details}
  \item{Tau}{ a matrix indicating which lags aregiven, see details  }
}
\details{
  \code{R} is a matrix of periodic autocovariances such that \eqn{R_k(l)}
  is the \eqn{(k,l+1)}th entry of the matrix \code{R}, see
  \code{\link{pcme-package}} for full description.

  \code{Tau} is a matrix of the same size as \code{R}. It specifies the
  missingness status of the elements of \code{R}.  \code{Tau[k,l] = 1}
  if \eqn{R_k(l-1)} is given, \code{Tau[k,l] = 0} if \eqn{R_k(l-1)} is
  missing and \eqn{(k,l-1)} is a gap, and \code{Tau[k,l] = 1} if
  \eqn{R_k(l-1)} is missing but \eqn{(k,l-1)} is not a gap. See
  \code{\link{pcme-package}} for the terminology and Boshnakov and
  Lambert-Lacroix (2009?) for details.
}
\value{
  For \code{LD}, a list with components \code{R}, \code{Beta}, and
  \code{sigma2f}, see the description below.

    For \code{LDgrad}, a list with components \code{gradsigma2f} and
    \code{sigma2f}.

    For \code{LDhessian}, a list with components \code{sigma2f},
    \code{gradsigma2f}, and \code{grad2sigma2f}.

  \item{R }{matrix of autocovariances, \eqn{R[k,l]=R_k(l-1)}.}
  \item{Beta}{matrix of partial autocorrelations, \eqn{Beta[k,l]=\beta_k(l-1)}.}
  \item{sigma2f}{The forward prediction variances.}
  \item{gradsigma2f}{The gradient of the forward prediction variances}
  \item{grad2sigma2f}{The Hessian of the forward prediction variances}
}
\references{
  Lambert-Lacroix, Sophie (2005) Extension of autocovariance
  coefficients sequence for periodically correlated processes.
  \emph{Journal of Time Series Analysis}, \bold{26}, No. 6, 423-435.
}
\author{ Sophie Lambert-Lacroix }
\note{
  The filter coefficients have the signs used by Lambert-Lacroix (2005).

  \eqn{R_k(l)} is set to 0 for \eqn{l>p_k} (??? check this!)

  Unlike higher level functions like \code{pcme}, the argument
  \code{tau} here has a rigid format and gaps cannot be
  specified by \code{NA}'s in \code{R}.
}
\seealso{
  \code{\link{pldinverse}}
}
\examples{
# From file LD.R
R <- matrix(c(1,1,2,0.9,0.8,0.7,0.4,0.5,0.6,0.9,0.9,0.9),nrow=3)
Tau <- matrix(c(1,1,1,1,0,0,1,1,2,1,2,2),nrow=3)
R[Tau==2] <- 0
LD(R,Tau)

R[,1]<-R[,1]+1
LD(R,Tau)


# From file LDgrad.r
R <- matrix(c(1,1,2,0.9,0.8,0.7,0.4,0.5,0.6,0.9,0.9,0.9),nrow=3)
Tau <- matrix(c(1,1,1,1,0,0,1,1,2,1,2,2),nrow=3)
R[,1]<-R[,1]+1
LD(R,Tau)


R <- matrix(c(1,1,2,0.9,0.8,0.7,0.4,0.5,0.6,0.9,0.9,0.9),nrow=3)
Tau <- matrix(c(1,1,1,0,1,1,1,1,1,0,2,2),nrow=3)
R[,1]<-R[,1]+1
LD(R,Tau)


# From file LDhessian.r
R <- matrix(c(1,1,2,0.9,0.8,0.7,0.4,0.5,0.6,0.9,0.9,0.9),nrow=3)
Tau <- matrix(c(1,1,1,1,0,0,1,1,2,1,2,2),nrow=3)
R[,1]<-R[,1]+1
LDhessian(R,Tau)


R <- matrix(c(1,1,2,0.9,0.8,0.7,0.4,0.5,0.6,0.9,0.9,0.9),nrow=3)
Tau <- matrix(c(1,1,1,0,1,1,1,1,1,0,2,2),nrow=3)
R[,1]<-R[,1]+1
LD(R,Tau)
}
\keyword{ ts }
