\name{pldinverse}
\alias{pldinverse}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Compute autocovariances from periodic partial autocorrelations }
\description{
  Compute periodic autocovariances from periodic partial
  autocorrelations using the inverse periodic Levinson-Durbin algorithm.
}
\usage{
pldinverse(beta)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{beta}{ numeric matrix, containing the periodic partial
    autocorrelations for lags 0, 1, \ldots, \code{ncol(beta)-1},
    see \code{\link{pcme-package}} for details.
  }
}
%\details{
%}
\value{
  A list containing the  following components:
  \item{R}{numeric matrix with the autocovariances.}
  \item{Af}{numeric matrix with the periodic filter coefficients.}
  \item{sigma2f}{numeric vector with the forward error variances.}
}
\references{
  Boshnakov, Georgi  and Lambert-Lacroix, Sophie (2009?)
  Maximum entropy for periodically correlated processes from
  nonconsecutive autocovariance coefficients.
  \emph{J. Time Series Anal.} (to appear)

  Lambert-Lacroix, Sophie (2005) Extension of autocovariance
  coefficients sequence for periodically correlated processes.
  \emph{Journal of Time Series Analysis}, \bold{26}, No. 6, 423-435.
}
\author{ Sophie Lambert-Lacroix }
\note{
  The signs of the coefficients of the filter are as in Lambert-Lacroix (2005).
}
\seealso{
  \code{\link{LD}}
}
\examples{
#Ex
beta <- matrix(c(3,2,0.6,0,0.85,0,-0.55,0,-0.55,0),nrow=2)
pldinverse(beta)

#Ex for ME method with gaps
#Ex1: We consider a PAR(1,2). The acf is computed for lag 0:3.
#We consider only one gap at season = 1 and lag = 2.
#By construction, we know that the solution is the one
#of the begining.
beta <- matrix(c(1,1,0.5,0.3,0,-0.3,0,0),nrow=2)
R1 <- pldinverse(beta)$R
gaps1 <- matrix(1,2,4)
gaps1[1,3] <- 0

#Ex2
beta <- matrix(c(1,1,0.9,0.3,0,-0.3,0,0),nrow=2)
R2 <- pldinverse(beta)$R
gaps2 <- matrix(1,2,4)
gaps2[1,3] <- 0

#Ex3: exemple nearly to the singularity
beta <- matrix(c(1,1,0.999999,0.3,0,-0.3,0,0),nrow=2)
R3 <- pldinverse(beta)$R
gaps3 <- matrix(1,2,4)
gaps3[1,3] <- 0

#Ex4: We consider a PAR(1,2) with singularity (one pacf coefficient equal
#to 0) and  compute the acf for lag 0:3.
#We consider only one gap at season = 1 and lag = 2.
#demontrer que la solution ME est bien degeneree
beta <- matrix(c(1,1,0.5,0.3,0,1,0,0),nrow=2)
R4 <- pldinverse(beta)$R
gaps4 <- matrix(1,2,4)
gaps4[1,3] <- 0

#Ex5: We construct one example for which there does not exist solution.
beta <- matrix(c(1,1,2,0.3),nrow=2)
R5 <- pldinverse(beta)$R
R5 <- cbind(R5,c(0.7,0.8),c(0.9,0.9))
gaps5 <- matrix(1,2,4)
gaps5[2,3] <- 0
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ts }
